@@ -22,7 +22,8 @@ int blacklist_init(char *whitelist, char *blacklist,
 		char **whitelist_entries,
 		size_t whitelist_entries_len,
 		char **blacklist_entries,
-		size_t blacklist_entries_len);
+		size_t blacklist_entries_len,
+        int ignore_invalid_hosts);
 
 uint64_t blacklist_count_allowed();
 
