@@ -1,5 +1,7 @@
 #ifdef __APPLE__
+#pragma GCC diagnostic ignored "-Wflexible-array-extensions"
 #include <dnet.h>
+#pragma GCC diagnostic warning "-Wflexible-array-extensions"
 #endif
 
 #ifndef __FAVOR_BSD
@@ -21,3 +23,6 @@
 #include <net/if.h>
 #include <ifaddrs.h>        // NOTE: net/if.h MUST be included BEFORE ifaddrs.h
 #include <arpa/inet.h>
+
+#define MAC_ADDR_LEN ETHER_ADDR_LEN
+#define UNUSED __attribute__((unused))
