@@ -11,4 +11,4 @@ void* xmalloc(size_t size);
 
 void* xrealloc(void *ptr, size_t size);
 
-#endif
\ No newline at end of file
+#endif /* ZMAP_ALLOC_H */
