@@ -1,3 +1,11 @@
+/*
+ * ZMap Copyright 2013 Regents of the University of Michigan
+ *
+ * Licensed under the Apache License, Version 2.0 (the "License"); you may not
+ * use this file except in compliance with the License. You may obtain a copy
+ * of the License at http://www.apache.org/licenses/LICENSE-2.0
+ */
+
 #ifndef ZMAP_TREE_H
 #define ZMAP_TREE_H
 
@@ -28,9 +36,9 @@ union node_value {
 	enum operation op;
 };
 
-typedef struct node {
-	struct node *left_child;
-	struct node *right_child;
+typedef struct node_st {
+	struct node_st *left_child;
+	struct node_st *right_child;
 	enum node_type type;
 	union node_value value;
 } node_t;
