@@ -15,6 +15,7 @@ extern output_module_t module_csv_file;
 
 #ifdef REDIS
 extern output_module_t module_redis;
+extern output_module_t module_redis_csv;
 extern output_module_t module_csv_redis;
 #endif
 
@@ -22,14 +23,21 @@ extern output_module_t module_csv_redis;
 extern output_module_t module_json_file;
 #endif
 
+#ifdef MONGODB
+extern output_module_t module_mongodb;
+#endif
+
 output_module_t* output_modules[] = {
 	&module_csv_file,
 #ifdef REDIS
 	&module_redis,
-	&module_csv_redis,
+	&module_redis_csv,
 #endif
 #ifdef JSON
-	&module_json_file
+	&module_json_file,
+#endif
+#ifdef MONGODB
+	&module_mongodb,
 #endif
 	// ADD YOUR MODULE HERE
 };
