@@ -30,7 +30,8 @@ int synscan_global_initialize(struct state_conf *state)
 }
 
 int synscan_init_perthread(void* buf, macaddr_t *src,
-		macaddr_t *gw, port_h_t dst_port)
+		macaddr_t *gw, port_h_t dst_port,
+		__attribute__((unused)) void **arg_ptr)
 {
 	memset(buf, 0, MAX_PACKET_SIZE);
 	struct ether_header *eth_header = (struct ether_header *) buf;
@@ -44,7 +45,7 @@ int synscan_init_perthread(void* buf, macaddr_t *src,
 }
 
 int synscan_make_packet(void *buf, ipaddr_n_t src_ip, ipaddr_n_t dst_ip,
-		uint32_t *validation, int probe_num)
+		uint32_t *validation, int probe_num, __attribute__((unused)) void *arg)
 {
 	struct ether_header *eth_header = (struct ether_header *)buf;
 	struct ip *ip_header = (struct ip*)(&eth_header[1]);
@@ -72,11 +73,11 @@ void synscan_print_packet(FILE *fp, void* packet)
 	struct ether_header *ethh = (struct ether_header *) packet;
 	struct ip *iph = (struct ip *) &ethh[1];
 	struct tcphdr *tcph = (struct tcphdr *) &iph[1];
-	fprintf(fp, "tcp { source: %u | dest: %u | seq: %u | checksum: %u }\n",
+	fprintf(fp, "tcp { source: %u | dest: %u | seq: %u | checksum: %#04X }\n",
 			ntohs(tcph->th_sport),
 			ntohs(tcph->th_dport),
 			ntohl(tcph->th_seq),
-			ntohl(tcph->th_sum));
+			ntohs(tcph->th_sum));
 	fprintf_ip_header(fp, iph);
 	fprintf_eth_header(fp, ethh);
 	fprintf(fp, "------------------------------------------------------\n");
