@@ -1,3 +1,11 @@
+/*
+ * ZMap Copyright 2013 Regents of the University of Michigan
+ *
+ * Licensed under the Apache License, Version 2.0 (the "License"); you may not
+ * use this file except in compliance with the License. You may obtain a copy
+ * of the License at http://www.apache.org/licenses/LICENSE-2.0
+ */
+
 #ifndef ZMAP_SEND_LINUX_H
 #define ZMAP_SEND_LINUX_H
 
@@ -13,19 +21,10 @@
 // Dummy sockaddr for sendto
 static struct sockaddr_ll sockaddr;
 
-
-int get_socket(void)
-{
-	int sock = socket(AF_PACKET, SOCK_RAW, htons(ETH_P_ALL));
-	if (sock <= 0) {
-		log_fatal("send", "couldn't create socket. "
-			  "Are you root? Error: %s\n", strerror(errno));
-	}
-	return sock;
-}
-
-int send_run_init(int sock)
+int send_run_init(sock_t s)
 {
+	// Get the actual socket
+	int sock = s.sock;
 	// get source interface index
 	struct ifreq if_idx;
 	memset(&if_idx, 0, sizeof(struct ifreq));
@@ -60,9 +59,9 @@ int send_run_init(int sock)
 	return EXIT_SUCCESS;
 }
 
-int send_packet(int fd, void *buf, int len)
+int send_packet(sock_t sock, void *buf, int len, UNUSED uint32_t idx)
 {
-	return sendto(fd, buf, len, 0,
+	return sendto(sock.sock, buf, len, 0,
 		      (struct sockaddr *) &sockaddr,
 		      sizeof(struct sockaddr_ll));
 }
