@@ -1,3 +1,11 @@
+/*
+ * ZMap Copyright 2013 Regents of the University of Michigan
+ *
+ * Licensed under the Apache License, Version 2.0 (the "License"); you may not
+ * use this file except in compliance with the License. You may obtain a copy
+ * of the License at http://www.apache.org/licenses/LICENSE-2.0
+ */
+
 #ifndef ZMAP_SHARD_H
 #define ZMAP_SHARD_H
 
@@ -11,6 +19,7 @@ typedef struct shard {
 	struct shard_state {
 		uint32_t sent;
 		uint32_t blacklisted;
+		uint32_t whitelisted;
 		uint32_t failures;
 		uint32_t first_scanned;
 		uint32_t max_targets;
